#!/usr/bin/env python
from __future__ import print_function

__author__ = "Mislav Novakovic <mislav.novakovic@sartura.hr>"
__copyright__ = "Copyright 2018, Deutsche Telekom AG"
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This sample application demonstrates use of Python programming language bindings for sysrepo library.
# Original c application was rewritten in Python to show similarities and differences
# between the two.
#
# Most notable difference is in the very different nature of languages, c is weakly statically typed language
# while Python is strongly dynamiclally typed. Python code is much easier to read and logic easier to comprehend
# for smaller scripts. Memory safety is not an issue but lower performance can be expected.
#
# The original c implementation is also available in the source, so one can refer to it to evaluate trade-offs.


import sysrepo as sr
import sys

# Function to print current configuration state.
# It does so by loading all the items of a session and printing them out.
def print_current_config(session, module_name):
    select_xpath = "/" + module_name + ":*//."

    values = session.get_items(select_xpath)

    for i in range(values.val_cnt()):
        print (values.val(i).to_string(),end='')

# Function to be called for subscribed client of given session whenever configuration changes.
def module_change_cb(sess, module_name, xpath, event, request_id, private_data):
    print ("\n\n ========== CONFIG HAS CHANGED, CURRENT RUNNING CONFIG: ==========\n")

    print_current_config(sess, module_name)

    return sr.SR_ERR_OK

# Notable difference between c implementation is using exception mechanism for open handling unexpected events.
# Here it is useful because `Conenction`, `Session` and `Subscribe` could throw an exception.
try:
    module_name = "ietf-interfaces"
    if len(sys.argv) > 1:
        module_name = sys.argv[1]
    else:
        print ("\nYou can pass the module name to be subscribed as the first argument")

    # connect to sysrepo
    conn = sr.Connection(sr.SR_CONN_DEFAULT)

    # start session
    sess = sr.Session(conn)

    # subscribe for changes in running config */
    subscribe = sr.Subscribe(sess)

    subscribe.module_change_subscribe(module_name, module_change_cb, None, None, 0, sr.SR_SUBSCR_DONE_ONLY)

    print ("\n\n ========== READING RUNNING CONFIG: ==========\n")
    try:
        print_current_config(sess, module_name)
    except Exception as e:
        print (e)

    sr.global_loop()
    
    subscribe.unsubscribe()

    sess.session_stop()

    conn=None

    print ("Application exit requested, exiting.\n")

except Exception as e:
    print (e)



#!/usr/bin/env python
from __future__ import print_function

# -*- coding: utf-8 -*-
__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from multiprocessing import Process, Manager
import os
import sys
import signal


class TestManager:
    """
    Class manages the testers and helps them to execute steps in synchronized order.
    Each tester is executed in a separeted process.
    """
    def __init__(self):
        self.manager = Manager()
        self.lock = self.manager.Lock()
        self.process_done = self.manager.Semaphore(0)
        self.queue = self.manager.Queue()
        self.sub_proc = self.manager.Queue()
        self._setup()

    def _setup(self):
        self.testers = []
        self.next_steps = []
        self.proc_ids = []
        self.subprocToKill = []

    def add_tester(self, tester):
        self.testers.append(tester)

    def start_processes(self, rand_sleep):
        """create process for each tester"""
        self.pids = self.manager.Array('l', range(len(self.testers)))

        for id in range(len(self.testers)):
            self.process_done.release()
            next_s = self.manager.Semaphore(0)

            p = Process(target=self.testers[id].run, args=(self.process_done, next_s, rand_sleep, self.lock, self.sub_proc, self.pids, id, self.queue))
            self.proc_ids.append(p)
            self.next_steps.append(next_s)
            p.start()
            self.pids[id] = p.pid

    def wait_for_processes(self):
        """wait for all process to finish"""
        for p in self.proc_ids:
            p.join()
            p.terminate()

        self.lock.acquire()
        self.lock.release()

    def run(self, rand_sleep=True):
        """Execute tester steps"""
        self.start_processes(rand_sleep)

        step = -1
        will_continue = range(len(self.next_steps))
        wait_for = range(len(self.next_steps))
        while True:
            if step >= 0:
                print("\n\n=================== TestManager step", step, "testers:", wait_for, file=sys.stderr)
            for _ in wait_for:
                self.process_done.acquire()
                if step >= 0:
                    proc, name, status = self.queue.get()
                    print(("Received ", proc, name, status), file=sys.stderr)
                    if status == True:
                        will_continue.append(proc)
                    elif isinstance(status, BaseException):
                        print("Error in tester", proc, name, "step", step)
                        for p in self.proc_ids:
                            p.terminate()
                        while not self.sub_proc.empty():
                            pid = self.sub_proc.get()
                            try:
                                os.kill(pid, signal.SIGKILL)
                            except:
                                pass
                        raise status

            if len(will_continue) == 0:
                break

            for id in will_continue:
                self.next_steps[id].release()

            wait_for = will_continue[:]
            will_continue = []
            step += 1

        self.wait_for_processes()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sysrepo as sr
import os

XP_TEST_MODULE_ENUM = "/test-module:main/enum"
XP_TEST_MODULE_ENUM_VALUE = "maybe"

XP_TEST_MODULE_RAW = "/test-module:main/raw"
XP_TEST_MODULE_RAW_VALUE = "SGVsbG8gd29ybGQh"

XP_TEST_MODULE_BITS = "/test-module:main/options"
XP_TEST_MODULE_BITS_VALUE = "strict"

XP_TEST_MODULE_BOOL = "/test-module:main/boolean"
XP_TEST_MODULE_BOOL_VALUE_T = True

XP_TEST_MODULE_DEC64 = "/test-module:main/dec64"
XP_TEST_MODULE_DEC64_VALUE_T = 9.85

XP_TEST_MODULE_EMPTY = "/test-module:main/empty"

XP_TEST_MODULE_IDREF = "/test-module:main/id_ref"
XP_TEST_MODULE_IDREF_VALUE = "id_1"

XP_TEST_MODULE_INT8 = "/test-module:main/i8"
XP_TEST_MODULE_INT8_VALUE_T = 8

XP_TEST_MODULE_INT16 = "/test-module:main/i16"
XP_TEST_MODULE_INT16_VALUE_T = 16

XP_TEST_MODULE_INT32 = "/test-module:main/i32"
XP_TEST_MODULE_INT32_VALUE_T = 32

XP_TEST_MODULE_INT64 = "/test-module:main/i64"
XP_TEST_MODULE_INT64_VALUE_T = 64

XP_TEST_MODULE_STRING = "/test-module:main/string"
XP_TEST_MODULE_STRING_VALUE = "str"

XP_TEST_MODULE_UINT8 = "/test-module:main/ui8"
XP_TEST_MODULE_UINT8_VALUE_T = 8

XP_TEST_MODULE_UINT16 = "/test-module:main/ui16"
XP_TEST_MODULE_UINT16_VALUE_T = 16

XP_TEST_MODULE_UINT32 = "/test-module:main/ui32"
XP_TEST_MODULE_UINT32_VALUE_T = 32

XP_TEST_MODULE_UINT64 = "/test-module:main/ui64"
XP_TEST_MODULE_UINT64_VALUE_T = 64

def remove_ietf_ip_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.remove_module("ietf-ip")
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None
    return True

def create_ietf_ip_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.install_module(os.environ['TESTS_DIR']+"/files/ietf-ip.yang",os.environ['TESTS_DIR']+"/files", [])
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None
    return True

def remove_iana_if_type_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.remove_module("iana-if-type")
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None
    return True

def create_iana_if_type_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.install_module(os.environ['TESTS_DIR']+"/files/iana-if-type.yang",os.environ['TESTS_DIR']+"/files", [])
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None
    return True

def remove_ietf_interfaces_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.remove_module("ietf-interfaces")
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None
    return True

def create_ietf_interfaces_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.install_module(os.environ['TESTS_DIR']+"/files/ietf-interfaces.yang",os.environ['TESTS_DIR']+"/files", [])
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None
    return True


def remove_referenced_data_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.remove_module("referenced-data")
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None
    return True

def create_referenced_data_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.install_module(os.environ['TESTS_DIR']+"/files/referenced-data.yang",os.environ['TESTS_DIR']+"/files", [])
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None
    return True

def remove_test_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.remove_module("test-module")
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None
    return True

def create_test_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.install_module(os.environ['TESTS_DIR']+"/files/test-module.yang",os.environ['TESTS_DIR']+"/files", [])
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None

    try:
        connection = sr.Connection(sr.SR_CONN_DEFAULT)
        session = sr.Session(connection, sr.SR_DS_STARTUP)

        delete_all_items_test(session)

        v = sr.Val(XP_TEST_MODULE_ENUM_VALUE, sr.SR_ENUM_T)
        session.set_item(XP_TEST_MODULE_ENUM, v)

        v = sr.Val(XP_TEST_MODULE_RAW_VALUE, sr.SR_BINARY_T)
        session.set_item(XP_TEST_MODULE_RAW, v)

        v = sr.Val(XP_TEST_MODULE_BITS_VALUE, sr.SR_BITS_T)
        session.set_item(XP_TEST_MODULE_BITS, v)

        v = sr.Val(XP_TEST_MODULE_BOOL_VALUE_T, sr.SR_BOOL_T)
        session.set_item(XP_TEST_MODULE_BOOL, v)

        v = sr.Val(format(XP_TEST_MODULE_DEC64_VALUE_T, '.2f'))
        session.set_item(XP_TEST_MODULE_DEC64, v)

        v = sr.Val(None, sr.SR_LEAF_EMPTY_T)
        session.set_item(XP_TEST_MODULE_EMPTY, v)

        v = sr.Val(XP_TEST_MODULE_IDREF_VALUE, sr.SR_IDENTITYREF_T)
        session.set_item(XP_TEST_MODULE_IDREF, v)

        v = sr.Val(XP_TEST_MODULE_STRING_VALUE, sr.SR_STRING_T)
        session.set_item(XP_TEST_MODULE_STRING, v)

        v = sr.Val(XP_TEST_MODULE_INT8_VALUE_T, sr.SR_INT8_T)
        session.set_item(XP_TEST_MODULE_INT8, v)

        v = sr.Val(XP_TEST_MODULE_INT16_VALUE_T, sr.SR_INT16_T)
        session.set_item(XP_TEST_MODULE_INT16, v)

        v = sr.Val(XP_TEST_MODULE_INT32_VALUE_T, sr.SR_INT32_T)
        session.set_item(XP_TEST_MODULE_INT32, v)

        v = sr.Val(XP_TEST_MODULE_INT64_VALUE_T, sr.SR_INT64_T)
        session.set_item(XP_TEST_MODULE_INT64, v)

        v = sr.Val(XP_TEST_MODULE_UINT8_VALUE_T, sr.SR_UINT8_T)
        session.set_item(XP_TEST_MODULE_UINT8, v)

        v = sr.Val(XP_TEST_MODULE_UINT16_VALUE_T, sr.SR_UINT16_T)
        session.set_item(XP_TEST_MODULE_UINT16, v)

        v = sr.Val(XP_TEST_MODULE_UINT32_VALUE_T, sr.SR_UINT32_T)
        session.set_item(XP_TEST_MODULE_UINT32, v)

        v = sr.Val(XP_TEST_MODULE_UINT64_VALUE_T, sr.SR_UINT64_T)
        session.set_item(XP_TEST_MODULE_UINT64, v)

        for num in [1, 2, 42]:
            v = sr.Val(num, sr.SR_UINT8_T)
            session.set_item("/test-module:main/numbers", v)


        v = sr.Val(None, sr.SR_LIST_T)
        session.set_item("/test-module:list[key='k1']", v)

        v = sr.Val("id_1", sr.SR_IDENTITYREF_T)
        session.set_item("/test-module:list[key='k1']/id_ref", v)

        v = sr.Val(None, sr.SR_CONTAINER_PRESENCE_T)
        session.set_item("/test-module:list[key='k1']/wireless", v)

        v = sr.Val(None, sr.SR_LIST_T)
        session.set_item("/test-module:list[key='k2']", v)

        v = sr.Val("id_2", sr.SR_IDENTITYREF_T)
        session.set_item("/test-module:list[key='k2']/id_ref", v)

        session.apply_changes()
        session.session_stop()

        connection=None
    except Exception as e:
        print(e)
        connection = None
        return False

def remove_example_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)

    try:
        connection.remove_module("example-module")
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None
    return True

def create_example_module():
    connection = sr.Connection(sr.SR_CONN_DEFAULT)
    try:
        connection.install_module(os.environ['TESTS_DIR']+"/files/example-module.yang",os.environ['TESTS_DIR']+"/files", [])
    except Exception as e:
        print(e)
        connection = None
        return False

    connection = None

    try:
        connection = sr.Connection(sr.SR_CONN_DEFAULT)
        session = sr.Session(connection, sr.SR_DS_STARTUP)

        delete_all_items_example(session)
        v = sr.Val("Leaf value", sr.SR_STRING_T)
        session.set_item("/example-module:container/list[key1='key1'][key2='key2']/leaf", v)
        session.apply_changes()
        session.session_stop()
        connection=None
        return True
    except Exception as e:
        print(e)
        connection = None
        return False


def create_ietf_interfaces():
    try:
        connection = sr.Connection(sr.SR_CONN_DEFAULT)

        session = sr.Session(connection, sr.SR_DS_STARTUP)
        delete_all_items_ietf(session)

        v = sr.Val("iana-if-type:ethernetCsmacd", sr.SR_IDENTITYREF_T)
        session.set_item("/ietf-interfaces:interfaces/interface[name='eth0']/type", v)

        v = sr.Val("Ethernet 0", sr.SR_STRING_T);
        session.set_item("/ietf-interfaces:interfaces/interface[name='eth0']/description", v)

        v = sr.Val(True, sr.SR_BOOL_T);
        session.set_item("/ietf-interfaces:interfaces/interface[name='eth0']/enabled", v)

        v = sr.Val(24, sr.SR_UINT8_T)
        session.set_item("/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/address[ip='192.168.2.100']/prefix-length", v)

        v = sr.Val(True, sr.SR_BOOL_T)
        session.set_item("/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/enabled", v)

        v = sr.Val(1500, sr.SR_UINT16_T)
        session.set_item("/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/mtu", v)

        v = sr.Val("iana-if-type:ethernetCsmacd", sr.SR_IDENTITYREF_T)
        session.set_item("/ietf-interfaces:interfaces/interface[name='eth1']/type", v)

        v = sr.Val("Ethernet 1", sr.SR_STRING_T);
        session.set_item("/ietf-interfaces:interfaces/interface[name='eth1']/description", v)

        v = sr.Val(True, sr.SR_BOOL_T)
        session.set_item("/ietf-interfaces:interfaces/interface[name='eth1']/ietf-ip:ipv4/enabled", v)

        v = sr.Val(1500, sr.SR_UINT16_T)
        session.set_item("/ietf-interfaces:interfaces/interface[name='eth1']/ietf-ip:ipv4/mtu", v)

        v = sr.Val("iana-if-type:ethernetCsmacd", sr.SR_IDENTITYREF_T)
        session.set_item("/ietf-interfaces:interfaces/interface[name='gigaeth0']/type", v)

        v = sr.Val("GigabitEthernet 0", sr.SR_STRING_T);
        session.set_item("/ietf-interfaces:interfaces/interface[name='gigaeth0']/description", v)

        v = sr.Val(False, sr.SR_BOOL_T);
        session.set_item("/ietf-interfaces:interfaces/interface[name='gigaeth0']/enabled", v)

        session.apply_changes()
        session.session_stop()

        connection=None
        return True
    except Exception as e:
        print(e)
        connection = None
        return False

def delete_all_items_test(session):

    values = session.get_items("/test-module:*//*")

    if values == None:
        return

    for i in range(values.val_cnt()):
        session.delete_item(values.val(i).xpath())
        session.apply_changes()

def delete_all_items_example(session):

    values = session.get_items("/example-module:*//*")

    if values == None:
        return

    for i in range(values.val_cnt()):
        session.delete_item(values.val(i).xpath())
        session.apply_changes()

def delete_all_items_ietf(session):

    values = session.get_items("/ietf-interfaces:*//*")

    if values == None:
        return

    for i in range(values.val_cnt()):
        session.delete_item(values.val(i).xpath())
        session.apply_changes()

if __name__ == "__main__":
    create_referenced_data_module()
    create_test_module()
    remove_test_module()
    remove_referenced_data_module()



#!/usr/bin/env python
from __future__ import print_function

__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import unittest
import TestModule
import sysrepo as sr

class SysrepoBasicTest(unittest.TestCase):

    def remove_modules(self):
        TestModule.remove_example_module()
        TestModule.remove_test_module()
        TestModule.remove_referenced_data_module()

    @classmethod
    def setUpClass(self):
        self.remove_modules(self)
        self.conn= sr.Connection(sr.SR_CONN_DEFAULT)

    @classmethod
    def tearDownClass(self):
        self.conn=None
        self.remove_modules(self)

    @classmethod
    def setUp(self):
        if not TestModule.create_referenced_data_module():
            self.remove_modules(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_test_module():
            self.remove_modules(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_example_module():
            self.remove_modules(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        self.session = sr.Session(self.conn, sr.SR_DS_STARTUP)
        self.conn= sr.Connection(sr.SR_CONN_DEFAULT)

    @classmethod
    def tearDown(self):
        self.remove_modules(self)
        self.session.session_stop()
        self.conn=None

    def test_logg_stderr(self):
        log = sr.Logs()
        log.set_stderr(sr.SR_LL_NONE)

    def test_get_item(self):
        item = self.session.get_item("/test-module:main/i32")
        self.assertEqual(item.xpath(), "/test-module:main/i32")
        self.assertEqual(item.type(), sr.SR_INT32_T)

    def test_list_schema(self):
        schemas = self.conn.get_module_info()

    def test_get_items(self):
        vals = self.session.get_items("/test-module:main")
        for i in range(vals.val_cnt()):
            v = vals.val(i)
            self.assertRegex(v.xpath(), "/test-module:main*")

    def test_set_item(self):
        xpath = "/example-module:container/list[key1='abc'][key2='def']/leaf"
        v = sr.Val("Hey hou", sr.SR_STRING_T)
        self.session.set_item(xpath, v)
        self.session.apply_changes()
        new_value = self.session.get_item(xpath)
        self.assertEqual(new_value.type(), sr.SR_STRING_T)
        self.assertEqual(new_value.data().get_string(), v.data().get_string())

    # This test used to put values to None in loop below. At the moment that segfaults.
    # It also did not add 'type' and 'full' name to xpath.
    def test_delete_item(self):
        lists = ["/test-module:user[name='A']", "/test-module:user[name='B']", "/test-module:user[name='C']" ]
        v = sr.Val("", sr.SR_STRING_T)
        for l in lists:
            self.session.set_item(l+'/type', v)
            self.session.set_item(l+'/full-name', v)
        self.session.apply_changes()
        items = self.session.get_items("/test-module:user")

        self.assertEqual(len(lists), items.val_cnt())
        for l in lists:
            self.session.delete_item(l)
        self.session.apply_changes()

    # Setting item to None segfaults...
    def test_move_item(self):
        lists = ["/test-module:user[name='A']", "/test-module:user[name='B']", "/test-module:user[name='C']" ]
        for l in lists:
            v = sr.Val(None, sr.SR_LIST_T)
            self.session.set_item(l, v)

        self.session.apply_changes()
        items = self.session.get_items("/test-module:user")
        for i in range(0, len(lists)):
            self.assertEqual(items.val(i).xpath(), lists[i])

        self.session.move_item(lists[1], sr.SR_MOVE_FIRST)

        lists.insert(0, lists.pop(1))

        items = self.session.get_items("/test-module:user")
        for i in range(0, len(lists)):
            self.assertEqual(items.val(i).xpath(), lists[i])

    def test_validate(self):
        v = sr.Val(42, sr.SR_UINT8_T)
        self.session.set_item("/test-module:main/numbers", v)

    def test_commit_empty(self):
        TestModule.create_test_module()
        v_old = self.session.get_item("/test-module:main/string")
        TestModule.delete_all_items_test(self.session)
        self.session.apply_changes()
        #test random leaf that was deleted
        v_none = self.session.get_item("/test-module:main/string")
        self.assertIsNone(v_none)
        self.session.set_item("/test-module:main/string", v_old)
        self.session.apply_changes()
        TestModule.create_test_module()

if __name__ == '__main__':
    unittest.main()



# !/usr/bin/env python
from csv import excel

__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import unittest
import TestModule
import sysrepo as sr

class MoveTest(unittest.TestCase):


    def remove_modules(self):
        TestModule.remove_test_module()
        TestModule.remove_referenced_data_module()

    @classmethod
    def setUpClass(self):
        self.remove_modules(self)

    @classmethod
    def tearDownClass(self):
        self.remove_modules(self)

    @classmethod
    def tearDown(self):
        self.remove_modules(self)

    @classmethod
    def setUp(self):
        if not TestModule.create_referenced_data_module():
            self.remove_modules(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_test_module():
            self.remove_modules(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        conn = sr.Connection(sr.SR_CONN_DEFAULT)
        session = sr.Session(conn, sr.SR_DS_STARTUP)
        session.delete_item("/test-module:user[name='A']")
        session.delete_item("/test-module:user[name='B']")
        session.delete_item("/test-module:user[name='C']")
        session.delete_item("/test-module:user[name='D']")
        session.apply_changes()
        session.set_item("/test-module:user[name='A']", sr.Val(None, sr.SR_LIST_T))
        session.set_item("/test-module:user[name='B']", sr.Val(None, sr.SR_LIST_T))
        session.set_item("/test-module:user[name='C']", sr.Val(None, sr.SR_LIST_T))
        session.set_item("/test-module:user[name='D']", sr.Val(None, sr.SR_LIST_T))
        session.apply_changes()

        session.session_stop()

        conn=None

    def compareListItems(self, items, expected):
        for i in range(len(expected)):
            self.assertEqual(items.val(i).xpath(), "/test-module:user[name='{0}']".format(expected[i]))

    def test_move_after_last(self):
        conn = sr.Connection(sr.SR_CONN_DEFAULT)
        self.session = sr.Session(conn, sr.SR_DS_STARTUP)
        items = self.session.get_items("/test-module:user")
        self.compareListItems(items, ["A", "B", "C", "D"])
        self.session.move_item("/test-module:user[name='B']", sr.SR_MOVE_AFTER, "[name='D']","B")
        self.session.apply_changes()
        items = self.session.get_items("/test-module:user")
        self.compareListItems(items, ["A", "C", "D", "B"])
        self.session.session_stop()

        conn=None

    def test_move_before_first(self):
        conn = sr.Connection(sr.SR_CONN_DEFAULT)
        self.session = sr.Session(conn, sr.SR_DS_STARTUP)
        self.session.move_item("/test-module:user[name='C']", sr.SR_MOVE_BEFORE, "[name='A']", "C")
        self.session.apply_changes()
        items = self.session.get_items("/test-module:user")
        self.compareListItems(items, ["C", "A", "B", "D"])
        self.session.session_stop()

        conn=None

    def test_move_after_unknown(self):
        conn = sr.Connection(sr.SR_CONN_DEFAULT)
        self.session = sr.Session(conn, sr.SR_DS_STARTUP)
        with self.assertRaises(RuntimeError):
            self.session.move_item("/test-module:user[name='B']", sr.SR_MOVE_AFTER, "[name='XY']", "B")
            self.session.apply_changes()
        self.session.session_stop()

        conn=None


    def test_move_last_first(self):
        conn = sr.Connection(sr.SR_CONN_DEFAULT)
        self.session = sr.Session(conn, sr.SR_DS_STARTUP)
        self.session.move_item("/test-module:user[name='C']", sr.SR_MOVE_LAST)
        items = self.session.get_items("/test-module:user")
        self.compareListItems(items, ["A", "B", "D", "C"])
        self.session.session_stop()

        conn=None

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ConcurrentHelpers import *
import TestModule
import sysrepo as sr

class CommitTester(SysrepoTester):
    def setup(self):
        super(CommitTester, self).setup()
        self.add_step(self.setItemStep)
        self.add_step(self.commitFailStep)
        self.add_step(self.checkItemStep)
        self.add_step(self.commitStep)
        self.add_step(self.checkItemStep)
        self.add_step(self.stopSession)
        self.add_step(self.disconnect)

    def setItemStep(self):
        v = sr.Val(99, sr.SR_INT8_T)
        self.session.set_item("/test-module:main/i8", v)

    def checkItemStep(self):
        v = self.session.get_item("/test-module:main/i8")
        self.tc.assertEqual(99, v.data().get_int8())

class CommitTest(unittest.TestCase):

    def remove_modules(self):
        TestModule.remove_test_module()
        TestModule.remove_referenced_data_module()

    @classmethod
    def setUpClass(self):
        self.remove_modules(self)

    @classmethod
    def tearDownClass(self):
        self.remove_modules(self)

    @classmethod
    def setUp(self):
        if not TestModule.create_referenced_data_module():
            self.remove_modules(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_test_module():
            self.remove_modules(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return

    @classmethod
    def tearDown(self):
        self.remove_modules(self)

    def test_CommitAndDsLock(self):
        tm = TestManager()

        first = SysrepoTester("First")
        first.add_step(first.lockStep)
        first.add_step(first.waitStep)
        first.add_step(first.unlockStep)
        first.add_step(first.stopSession)
        first.add_step(first.disconnect)
        tm.add_tester(first)

        tm.add_tester(CommitTester("Second"))
        tm.run()


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
from __future__ import print_function

__author__ = "Antonio Paunovic <antonio.paunovic@sartura.hr>"
__copyright__ = "Copyright 2018, Deutsche Telekom AG"
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sysrepo as sr
import sys
import os
from time import sleep
from datetime import datetime

# def print_val(val, fd=sys.stdout):
#     print(val+'|', file=fd, end='')

# Helper function for printing changes given operation, old and new value.
def print_change(fd, op, old_val, new_val):
    if (op == sr.SR_OP_CREATED):
           print("CREATED|", file=fd, end='')
           print("CREATED|", file=sys.stderr, end='')
           print(new_val.to_string().replace(' = ', '|').replace(' ', '|'), file=fd, end='')
    elif (op == sr.SR_OP_DELETED):
           print("DELETED|", file=fd, end='')
           vstr = old_val.to_string().replace(' = ', '|').replace(' ', '|')
           print(vstr, file=fd, end='')
    elif (op == sr.SR_OP_MODIFIED):
           print("MODIFIED|", file=sys.stderr, end='')
           print("MODIFIED|", file=fd, end='')

           vstr=old_val.to_string().replace(' = ', '|').replace(' ', '|')+'|'+new_val.to_string().replace(' = ', '|').replace(' ', '|')
           vstr = vstr.replace('\n', '')
           print(vstr, file=fd, end='')
    elif (op == sr.SR_OP_MOVED):
            print('MOVE', file=sys.stderr, end='')
            print("MOVED|" + new_val.xpath() + "|" + old_val.xpath(), file=fd, end='')


# Function to be called for subscribed client of given session whenever configuration changes.
def module_change_cb(sess, module_name, xpath, event, request_id, private_data):

    settings = private_data
    if not 'module_name' in settings:
        print >> sys.stderr, "private_data error", private_data
        return sr.SR_ERR_INTERNAL
    if not 'xpath' in settings:
        print >> sys.stderr, "private_data error", private_data
        return sr.SR_ERR_INTERNAL


    out = open(settings['filename'], 'w') if 'filename' in settings else sys.stdout

    if out is None:
        print('File', settings['filename'], 'could not be opened')
        return sr.SR_ERR_INTERNAL

    change_path = "/" + module_name + ":*//."
    try:
        it = sess.get_changes_iter(change_path);
        if it is None:
            print("Get changes iter failed for path", change_path, file=sys.stderr)
            return sr.SR_ERR_OK

        while True:
            try:
                change = sess.get_change_next(it)
                if change == None:
                    break
            except Exception as e:
                break
            print_change(out, change.oper(), change.old_val(), change.new_val())

    except Exception as e:
        pass

    out.close()

    return sr.SR_ERR_OK

if __name__ == "__main__":

    if len(sys.argv) < 3:
        print("Usage: %s module_name xpath [output_file]\n\n")
        sys.exit(1)

    settings = { 'module_name' : sys.argv[1], 'xpath' : sys.argv[2], 'filename' : sys.argv[3] }
    # connect to sysrepo
    conn = sr.Connection(sr.SR_CONN_DEFAULT)

    # start session
    sess = sr.Session(conn)

    # subscribe for changes in running config */
    subscribe = sr.Subscribe(sess)

    # Subscribe for changes in running config - try multiple times because in test
    # multiple client might try to subscribe to the same model
    for i in range(3):
        try:
            subscribe.module_change_subscribe(settings['module_name'], module_change_cb, settings['xpath'], settings, 0, sr.SR_SUBSCR_DONE_ONLY)
        except Exception as e:
            # Multiple clients might try to subscribe to the same model.
            sleep(10.0/10e6)
            continue
        break
    with open("pipe_"+settings['filename'], "w") as fifo:
        fifo.write("subscribed")
    sr.global_loop()
    subscribe.unsubscribe()

    sess.session_stop()
    
    conn = None

    print("Application exit requested, exiting.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ConcurrentHelpers import *
import signal
import os
import errno
import subprocess
import TestModule
import sysrepo as sr
import unittest
from time import sleep

class SubscriptionTester(SysrepoTester):

    def subscribeStep(self):
        # wait for running data file to be copied

        try:
            os.mkfifo("pipe_subscription_test")
        except OSError as oe:
            if oe.errno != errno.EEXIST:
                raise

        self.process = subprocess.Popen(['python3','SubscriptionTestApp.py', format(os.getpid())])
        self.report_pid(self.process.pid)
        output = ""
        with open("pipe_subscription_test", "r") as fifo:
            output = fifo.readline()

        os.unlink("pipe_subscription_test")
        self.tc.assertEqual(str(output), "subscribed")

    def cancelSubscriptionStep(self):
        os.kill(self.process.pid, signal.SIGUSR1)
        self.process.wait()

    def killStep(self):
        os.kill(self.process.pid, signal.SIGTERM)
        self.process.wait()

    def getOperationalData(self, xpath, expected):
        if(self.session):
            vals = self.session.get_items(xpath)
            self.tc.assertEqual(len(expected), vals.val_cnt())
            for i in range(len(expected)):
                self.tc.assertEqual(vals.val(i).xpath(), expected[i])


class SubscriptionTest(unittest.TestCase):

    def remove_interfaces(self):
        TestModule.remove_ietf_interfaces_module()
        TestModule.remove_iana_if_type_module()
        TestModule.remove_ietf_ip_module()
    @classmethod
    def setUpClass(self):
        self.remove_interfaces(self)

    @classmethod
    def tearDownClass(self):
        self.remove_interfaces(self)

    @classmethod
    def setUp(self):
        if not TestModule.create_ietf_interfaces_module():
            self.remove_interfaces(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_iana_if_type_module():
            self.remove_interfaces(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_ietf_ip_module():
            self.remove_interfaces(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_ietf_interfaces():
            self.remove_interfaces(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return

    @classmethod
    def tearDown(self):
        self.remove_interfaces(self)


    def test_SubscribeUnsubscribe(self):
        tm = TestManager()

        reader = SubscriptionTester("First", sr.SR_DS_OPERATIONAL)
        subscriber = SubscriptionTester("Second")

        reader.add_step(reader.getOperationalData, "/ietf-interfaces:interfaces-state/interface[name='eth100']", [])
        subscriber.add_step(subscriber.waitStep)

        reader.add_step(reader.waitStep)
        subscriber.add_step(subscriber.subscribeStep)

        reader.add_step(reader.getOperationalData, "/ietf-interfaces:interfaces-state/interface[name='eth100']", ["/ietf-interfaces:interfaces-state/interface[name='eth100']"])
        subscriber.add_step(subscriber.waitStep)

        reader.add_step(reader.waitStep)
        subscriber.add_step(subscriber.cancelSubscriptionStep)

        reader.add_step(reader.getOperationalData, "/ietf-interfaces:interfaces-state/interface[name='eth100']", [])
        subscriber.add_step(subscriber.waitStep)

        reader.add_step(reader.stopSession)
        subscriber.add_step(subscriber.stopSession)

        reader.add_step(reader.disconnect)
        subscriber.add_step(subscriber.disconnect)

        tm.add_tester(reader)
        tm.add_tester(subscriber)
        tm.run()

    def test_SubscribeKill(self):
        tm = TestManager()

        reader = SubscriptionTester("First", sr.SR_DS_OPERATIONAL)
        subscriber = SubscriptionTester("Second")

        reader.add_step(reader.getOperationalData, "/ietf-interfaces:interfaces-state/interface[name='eth100']", [])
        subscriber.add_step(subscriber.waitStep)

        reader.add_step(reader.waitStep)
        subscriber.add_step(subscriber.subscribeStep)

        reader.add_step(reader.getOperationalData, "/ietf-interfaces:interfaces-state/interface[name='eth100']", ["/ietf-interfaces:interfaces-state/interface[name='eth100']"])
        subscriber.add_step(subscriber.waitStep)

        reader.add_step(reader.waitStep)
        subscriber.add_step(subscriber.cancelSubscriptionStep)

        reader.add_step(reader.getOperationalData, "/ietf-interfaces:interfaces-state/interface[name='eth100']", [])
        subscriber.add_step(subscriber.waitStep)

        reader.add_step(reader.stopSession)
        subscriber.add_step(subscriber.stopSession)

        reader.add_step(reader.disconnect)
        subscriber.add_step(subscriber.disconnect)

        tm.add_tester(reader)
        tm.add_tester(subscriber)
        tm.run()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ConcurrentHelpers import *
import TestModule
import sysrepo as sr


class User2(SysrepoTester):
    def __init__(self, name):
        super(User2, self).__init__()

    def setup(self):
        super(User2, self).setup()

        self.add_step(self.waitStep)
        self.add_step(self.lockFailStep)
        self.add_step(self.waitStep)
        self.add_step(self.lockStep)
        self.add_step(self.unlockStep)
        self.add_step(self.stopSession)
        self.add_step(self.disconnect)


class LockingTest(unittest.TestCase):

    def remove_modules(self):
        TestModule.remove_example_module()

    @classmethod
    def setUpClass(self):
        self.remove_modules(self)

    @classmethod
    def tearDownClass(self):
        self.remove_modules(self)

    @classmethod
    def setUp(self):
        if not TestModule.create_example_module():
            self.remove_modules(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return

    @classmethod
    def tearDown(self):
        self.remove_modules(self)

    def test_ConcurrentDataStoreLocking(self):
        tm = TestManager()

        first = SysrepoTester("First")
        first.add_step(first.lockStep)
        first.add_step(first.waitStep)
        first.add_step(first.unlockStep)
        first.add_step(first.stopSession)
        first.add_step(first.disconnect)
        tm.add_tester(first)

        tm.add_tester(User2("Second"))
        tm.run()

    def test_ConcurrentModelLocking(self):
        tm = TestManager()

        first = SysrepoTester("First")
        second = SysrepoTester("Second")

        first.add_step(first.lockModelStep, "example-module")
        second.add_step(second.waitStep)

        first.add_step(first.waitStep)
        second.add_step(second.lockFailModelStep, "example-module")

        first.add_step(first.unlockModelStep, "example-module")
        second.add_step(second.waitStep)

        second.add_step(second.lockModelStep, "example-module")
        second.add_step(second.unlockModelStep, "example-module")

        first.add_step(first.stopSession)
        second.add_step(second.stopSession)

        first.add_step(first.disconnect)
        second.add_step(second.disconnect)


        tm.add_tester(first)
        tm.add_tester(second)
        tm.run()

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ConcurrentHelpers import *
import subprocess
import TestModule
import sysrepo


class SysrepoModuleTester(SysrepoTester):
    sysrepoctl = "sysrepoctl"

    def installModuleStep(self, schema_path):
        rc = self.sr.install_module(os.environ['TESTS_DIR']+"/files/" + schema_path, os.environ['TESTS_DIR']+"/files", [])
        self.tc.assertEqual(rc, None)

    def uninstallModuleFailStep(self, module_name):
        rc = self.sr.remove_module(module_name)
        self.tc.assertNotEqual(rc, None)

    def uninstallModuleStep(self, module_name, log_level = sr.SR_LL_INF):
        rc = self.sr.remove_module(module_name)
        self.tc.assertEqual(rc, None)


class SchemasManagementTest(unittest.TestCase):

    def remove_modules(self):
        TestModule.remove_test_module()
        TestModule.remove_referenced_data_module()

    @classmethod
    def setUpClass(self):
        self.remove_modules(self)

    @classmethod
    def tearDownClass(self):
        self.remove_modules(self)

    @classmethod
    def setUp(self):
        if not TestModule.create_referenced_data_module():
            self.remove_modules(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_test_module():
            self.remove_modules(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return

    @classmethod
    def tearDown(self):
        self.remove_modules(self)

    def test_ModuleLoading(self):
         """Schemas are loaded on demand. Try to send multiple requests targeting the same model
         simultaneously. All of the should receive correct data.
         """
         tm = TestManager()
         tester1 = SysrepoTester("First", sr.SR_DS_STARTUP)
         tester2 = SysrepoTester("Second", sr.SR_DS_STARTUP)
         tester3 = SysrepoTester("Third", sr.SR_DS_STARTUP)
         tester4 = SysrepoTester("Fourth", sr.SR_DS_STARTUP)

         tester1.add_step(tester1.getItemsStepExpectedCount,
                          "/test-module:main/*", 19)
         tester2.add_step(tester2.getItemsStepExpectedCount,
                          "/test-module:main/*", 19)
         tester3.add_step(tester3.getItemsStepExpectedCount,
                          "/test-module:main/*", 19)
         tester4.add_step(tester4.getItemsStepExpectedCount,
                          "/test-module:main/*", 19)

         tester1.add_step(tester1.stopSession)
         tester2.add_step(tester2.stopSession)
         tester3.add_step(tester3.stopSession)
         tester4.add_step(tester4.stopSession)

         tester1.add_step(tester1.disconnect)
         tester2.add_step(tester2.disconnect)
         tester3.add_step(tester3.disconnect)
         tester4.add_step(tester4.disconnect)

         tm.add_tester(tester1)
         tm.add_tester(tester2)
         tm.add_tester(tester3)
         tm.add_tester(tester4)
         tm.run()

    def test_module_uninstall(self):
        test_module_file = "test-module.yang"
        referenced_data_file = "referenced-data.yang"
        file_location = "/tmp/"
        tm = TestManager()

        tester1 = SysrepoTester("First", sr.SR_DS_STARTUP)
        tester2 = SysrepoTester("Second", sr.SR_DS_STARTUP)
        tester3 = SysrepoTester("Third", sr.SR_DS_STARTUP)
        admin = SysrepoModuleTester()

        tester1.add_step(tester1.getItemsStepExpectedCount, "/test-module:main/*", 19)
        tester2.add_step(tester2.setItemStep, "/test-module:main/string", sr.Val("abcd", sr.SR_STRING_T))
        tester3.add_step(tester3.waitStep)
        tester1.add_step(tester1.waitStep)
        admin.add_step(admin.waitStep)

        tester1.add_step(tester1.commitStep)
        tester2.add_step(tester2.commitStep)
        tester3.add_step(tester3.waitStep)
        admin.add_step(admin.waitStep)

        admin.add_step(admin.stopSession)
        tester1.add_step(tester1.stopSession)
        tester2.add_step(tester2.stopSession)
        tester3.add_step(tester3.waitStep)

        admin.add_step(admin.disconnect)
        tester1.add_step(tester1.disconnect)
        tester2.add_step(tester2.disconnect)
        tester3.add_step(tester3.waitStep)

        admin.add_step(admin.waitStep)
        tester3.add_step(tester3.stopSession)

        tester3.add_step(tester3.uninstallModuleStep, "test-module")
        admin.add_step(admin.waitStep)

        admin.add_step(admin.waitStep)
        tester3.add_step(tester3.disconnect)

        admin.add_step(admin.restartConnection)
        tester3.add_step(tester3.restartConnection )

        admin.add_step(admin.waitStep)
        tester3.add_step(tester3.setItemFailStep, "/test-module:main/string", sr.Val("abcd", sr.SR_STRING_T))

        tester3.add_step(tester3.commitStep)
        admin.add_step(admin.waitStep)

        admin.add_step(admin.installModuleStep, test_module_file)
        tester3.add_step(tester3.waitStep)

        admin.add_step(admin.stopSession)
        tester3.add_step(tester3.stopSession)

        admin.add_step(admin.disconnect)
        tester3.add_step(tester3.disconnect)

        admin.add_step(admin.restartConnection)
        tester3.add_step(tester3.restartConnection)

        admin.add_step(admin.waitStep)
        tester3.add_step(tester3.setItemStep, "/test-module:main/string", sr.Val("abcd", sr.SR_STRING_T))

        admin.add_step(admin.stopSession)
        tester3.add_step(tester3.stopSession)

        admin.add_step(admin.disconnect)
        tester3.add_step(tester3.disconnect)

        tm.add_tester(tester1)
        tm.add_tester(tester2)
        tm.add_tester(tester3)
        tm.add_tester(admin)
        tm.run()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ConcurrentHelpers import *
from time import sleep
import sysrepo as sr


class SysrepoTester(Tester):
    def __init__(self, name="SysrepoUser", ds=sr.SR_DS_STARTUP, conn_mode=sr.SR_CONN_DEFAULT, connectInSetup=True):
        super(SysrepoTester, self).__init__(name)
        self.ds = ds
        self.conn = conn_mode
        self.autoconnect = connectInSetup

    def setup(self):
        if self.autoconnect:
            self.sr = sr.Connection(self.conn)
            self.session = sr.Session(self.sr, self.ds)

    def restartConnection(self):
        try:
            self.sr = sr.Connection(self.conn)
        except RuntimeError as r:
            raise r
        self.session = sr.Session(self.sr, self.ds)

    def disconnect(self):
        self.sr=None

    def stopSession(self):
        self.session.session_stop()
        self.session = None

    def startSession(self):
        self.tc.assertNotEqual(self.sr, None)
        self.tc.assertEqual(self.session, None)
        self.session = sr.Session(self.sr, self.ds)

    def lockStep(self):
        self.session.lock()

    def lockFailStep(self):
        with self.tc.assertRaises(RuntimeError):
            self.session.lock()

    def unlockStep(self):
        self.session.unlock()

    def lockModelStep(self, module_name):
        self.session.lock(module_name)

    def lockFailModelStep(self, module_name):
        with self.tc.assertRaises(RuntimeError):
            self.session.lock(module_name)

    def unlockModelStep(self, module_name):
        self.session.unlock(module_name)

    def commitFailStep(self):
        with self.tc.assertRaises(RuntimeError):
            self.session.apply_changes()

    def commitStep(self):
        self.session.apply_changes()

    def getItemsStep(self, xpath):
        self.session.get_items(xpath)
        self.session.get_items(xpath)

    def getItemsStepExpectedCount(self, xpath, count):
        items = self.session.get_items(xpath)
        self.tc.assertEqual(items.val_cnt(), count)

    def getItemsFailStep(self, xpath):
        with self.tc.assertRaisesRegex(RuntimeError, ".* found"):
            vs = self.session.get_items(xpath)
            if vs is None:
                raise (RuntimeError(".* found"))

    def deleteItemStep(self, xpath):
        self.session.delete_item(xpath)

    def setItemStep(self, xpath, value):
        self.session.set_item(xpath, value)

    def setItemFailStep(self, xpath, value):
        with self.tc.assertRaises(RuntimeError):
            self.session.set_item(xpath, value)

    def waitTimeoutStep(self, timeout):
        sleep(timeout)

    def uninstallModuleStep(self, module_name):
        rc = self.sr.remove_module(module_name)
        self.tc.assertEqual(rc, None)

    def uninstallModuleFailStep(self, module_name):
        rc = self.sr.remove_module(module_name)
        self.tc.assertNotEqual(rc, None)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ConcurrentHelpers import *
from random import randint
import signal
import os
import errno
import subprocess
import TestModule
import sysrepo as sr


class NotificationTester(SysrepoTester):

    def cleanup(self):
        if self.filename and os.path.isfile(self.filename):
            os.unlink(self.filename)

    def subscribeStep(self, module_name, xpath):
        self.filename = "notifications_test_" + str(randint(0, 9999))
        try:
            os.mkfifo("pipe_"+self.filename)
        except OSError as oe:
            if oe.errno != errno.EEXIST:
                raise

        self.process = subprocess.Popen(
            ['python3','NotificationTestApp.py', module_name, xpath, self.filename])
        self.report_pid(self.process.pid)
        # wait for running data file to be copied
        output = ""
        with open("pipe_"+self.filename, "r") as fifo:
            output = fifo.readline()

        os.unlink("pipe_"+self.filename)
        self.tc.assertEqual(str(output), "subscribed")

    def cancelSubscriptionStep(self):
        os.kill(self.process.pid, signal.SIGINT)
        self.process.wait()

    def checkNotificationStep(self, expected):
        try:
            with open(self.filename, "r") as f:
                self.notifications = []
                for line in f:
                    self.notifications.append(line.split("|"))

            self.tc.assertEqual(len(expected), len(self.notifications))

            ex_sorted = sorted(expected, key=lambda e: e[1])
            notif_sorted = sorted(self.notifications, key=lambda e: e[1])
            for i in range(len(expected)):
                self.tc.assertEqual(ex_sorted[i][0], notif_sorted[i][0])
                self.tc.assertEqual(ex_sorted[i][1], notif_sorted[i][1])
        except Exception as e:
            print(e)

    def checkNoNotificationArrived(self):
        self.tc.assertFalse(os.path.isfile(self.filename))

    def deleteNotifications(self):
        if os.path.isfile(self.filename):
            os.unlink(self.filename)


class NotificationTest(unittest.TestCase):


    def remove_interfaces(self):
        TestModule.remove_ietf_interfaces_module()
        TestModule.remove_iana_if_type_module()
        TestModule.remove_ietf_ip_module()
        TestModule.remove_example_module()
    @classmethod
    def setUpClass(self):
        self.remove_interfaces(self)

    @classmethod
    def tearDownClass(self):
        self.remove_interfaces(self)

    @classmethod
    def setUp(self):
        if not TestModule.create_ietf_interfaces_module():
            self.remove_interfaces(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_iana_if_type_module():
            self.remove_interfaces(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_ietf_ip_module():
            self.remove_interfaces(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_ietf_interfaces():
            self.remove_interfaces(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return
        if not TestModule.create_example_module():
            self.remove_interfaces(self)
            self.skipTest(self,"Test environment is not clean!")
            print("Environment is not clean!")
            return

    @classmethod
    def tearDown(self):
        self.remove_interfaces(self)

    def test_notify_delete(self):
        tm = TestManager()

        tester = SysrepoTester("Tester", sr.SR_DS_RUNNING)
        subscriber = NotificationTester("Subscriber")
        subscriber2 = NotificationTester("Subscriber2")
        subscriber3 = NotificationTester("Subscriber3")

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.subscribeStep,
                            "ietf-interfaces", "/ietf-interfaces:interfaces")
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.subscribeStep,
                             "example-module", "/example-module:container")

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.subscribeStep,
                             "ietf-interfaces", "/ietf-interfaces:interfaces/interface/ietf-ip:ipv4/address")
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(tester.deleteItemStep, "/ietf-interfaces:interfaces/interface[name='eth0']")
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(tester.commitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.checkNotificationStep,
                            [["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']"],
                             ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/name"],
                                ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/type"],
                                ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/enabled"],
                                ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/description"],
                                ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4"],
                                ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/address[ip='192.168.2.100']"],
                                ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/address[ip='192.168.2.100']/ip"],
                                ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/address[ip='192.168.2.100']/prefix-length"],
                                ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/mtu"],
                                ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/enabled"],
                                ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/forwarding"]])
        subscriber2.add_step(subscriber2.checkNotificationStep,
                              [["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/address[ip='192.168.2.100']"],
                               ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/address[ip='192.168.2.100']/ip"],
                                  ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/address[ip='192.168.2.100']/prefix-length"],
                               ])
        subscriber3.add_step(subscriber3.checkNoNotificationArrived)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.cancelSubscriptionStep)
        subscriber2.add_step(subscriber2.cancelSubscriptionStep)
        subscriber3.add_step(subscriber3.cancelSubscriptionStep)

        tester.add_step(tester.stopSession)
        subscriber.add_step(subscriber.stopSession)
        subscriber2.add_step(subscriber2.stopSession)
        subscriber3.add_step(subscriber3.stopSession)

        tester.add_step(tester.disconnect)
        subscriber.add_step(subscriber.disconnect)
        subscriber2.add_step(subscriber2.disconnect)
        subscriber3.add_step(subscriber3.disconnect)

        tm.add_tester(tester)
        tm.add_tester(subscriber)
        tm.add_tester(subscriber2)
        tm.add_tester(subscriber3)
        tm.run()

    def test_notify_modify(self):
        tm = TestManager()

        tester = SysrepoTester("Tester", sr.SR_DS_RUNNING, False)
        subscriber = NotificationTester("Subscriber")
        subscriber2 = NotificationTester("Subscriber2")
        subscriber3 = NotificationTester("Subscriber3")

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.subscribeStep, "ietf-interfaces",
                            "/ietf-interfaces:interfaces")
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.subscribeStep, "example-module",
                             "/example-module:container")

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.subscribeStep, "ietf-interfaces",
                             "/ietf-interfaces:interfaces/interface/ietf-ip:ipv4")
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(
            tester.setItemStep, "/ietf-interfaces:interfaces/interface[name='eth0']/type", sr.Val("iana-if-type:ethernetCsmacd", sr.SR_IDENTITYREF_T))
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(
            tester.setItemStep, "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/enabled", sr.Val(False, sr.SR_BOOL_T))
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(tester.commitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.checkNotificationStep,
                            [["MODIFIED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/enabled"]])
        subscriber2.add_step(subscriber2.checkNotificationStep,
                             [["MODIFIED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/enabled"]])
        subscriber3.add_step(subscriber3.checkNoNotificationArrived)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.cancelSubscriptionStep)
        subscriber2.add_step(subscriber2.cancelSubscriptionStep)
        subscriber3.add_step(subscriber3.cancelSubscriptionStep)

        tester.add_step(tester.stopSession)
        subscriber.add_step(subscriber.stopSession)
        subscriber2.add_step(subscriber2.stopSession)
        subscriber3.add_step(subscriber3.stopSession)

        tester.add_step(tester.disconnect)
        subscriber.add_step(subscriber.disconnect)
        subscriber2.add_step(subscriber2.disconnect)
        subscriber3.add_step(subscriber3.disconnect)

        tm.add_tester(tester)
        tm.add_tester(subscriber)
        tm.add_tester(subscriber2)
        tm.add_tester(subscriber3)
        tm.run()

    def test_notify_changes_in_multiple_modules(self):
        tm = TestManager()

        tester = SysrepoTester("Tester", sr.SR_DS_RUNNING, False)
        subscriber = NotificationTester("Subscriber")
        subscriber2 = NotificationTester("Subscriber2")
        subscriber3 = NotificationTester("Subscriber3")

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.subscribeStep, "ietf-interfaces",
                            "/ietf-interfaces:interfaces")
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.subscribeStep, "example-module",
                             "/example-module:container")

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.subscribeStep, "ietf-interfaces",
                             "/ietf-interfaces:interfaces/interface/ietf-ip:ipv4/address")
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(
            tester.setItemStep, "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/enabled", sr.Val(False, sr.SR_BOOL_T))
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(
            tester.setItemStep, "/example-module:container/list[key1='abc'][key2='def']/leaf", sr.Val("new value", sr.SR_STRING_T))
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(tester.commitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(tester.deleteItemStep,
                        "/example-module:container/list[key1='key1'][key2='key2']/leaf")
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(tester.commitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.checkNotificationStep,
                            [["MODIFIED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/enabled"]])
        subscriber2.add_step(subscriber2.checkNoNotificationArrived)
        subscriber3.add_step(subscriber3.checkNotificationStep, [
            ["DELETED", "/example-module:container/list[key1='key1'][key2='key2']/leaf"],
            ["CREATED", "/example-module:container/list[key1='abc'][key2='def']"],
            ["CREATED", "/example-module:container/list[key1='abc'][key2='def']/key1"],
            ["CREATED", "/example-module:container/list[key1='abc'][key2='def']/key2"],
            ["CREATED", "/example-module:container/list[key1='abc'][key2='def']/leaf"]
        ])

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.cancelSubscriptionStep)
        subscriber2.add_step(subscriber2.cancelSubscriptionStep)
        subscriber3.add_step(subscriber3.cancelSubscriptionStep)

        tester.add_step(tester.stopSession)
        subscriber.add_step(subscriber.stopSession)
        subscriber2.add_step(subscriber2.stopSession)
        subscriber3.add_step(subscriber3.stopSession)

        tester.add_step(tester.disconnect)
        subscriber.add_step(subscriber.disconnect)
        subscriber2.add_step(subscriber2.disconnect)
        subscriber3.add_step(subscriber3.disconnect)

        tm.add_tester(tester)
        tm.add_tester(subscriber)
        tm.add_tester(subscriber2)
        tm.add_tester(subscriber3)
        tm.run()

    def test_notify_same_path(self):
        """
        Four testers are subscribed for the same notifications. The changes
        are generated when the request from the first of them arrives.
        """
        tm = TestManager()

        tester = SysrepoTester("Tester", sr.SR_DS_RUNNING, False)
        subscriber = NotificationTester("Subscriber")
        subscriber2 = NotificationTester("Subscriber2")
        subscriber3 = NotificationTester("Subscriber3")
        subscriber4 = NotificationTester("Subscriber4")

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.subscribeStep, "ietf-interfaces",
                            "/ietf-interfaces:interfaces")
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)
        subscriber4.add_step(subscriber4.waitStep)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.subscribeStep, "ietf-interfaces",
                             "/ietf-interfaces:interfaces")
        subscriber3.add_step(subscriber3.waitStep)
        subscriber4.add_step(subscriber4.waitStep)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.subscribeStep, "ietf-interfaces",
                             "/ietf-interfaces:interfaces")
        subscriber4.add_step(subscriber4.waitStep)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)
        subscriber4.add_step(subscriber4.subscribeStep, "ietf-interfaces",
                             "/ietf-interfaces:interfaces")

        tester.add_step(tester.deleteItemStep,
                        "/ietf-interfaces:interfaces/interface[name='eth0']")
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)
        subscriber4.add_step(subscriber4.waitStep)

        tester.add_step(tester.commitStep)
        subscriber.add_step(subscriber.waitStep)
        subscriber2.add_step(subscriber2.waitStep)
        subscriber3.add_step(subscriber3.waitStep)
        subscriber4.add_step(subscriber4.waitStep)

        expected_changes = [["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/name"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/type"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/description"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/enabled"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/address[ip='192.168.2.100']"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/address[ip='192.168.2.100']/ip"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/address[ip='192.168.2.100']/prefix-length"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/enabled"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/mtu"],
                            ["DELETED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/forwarding"]]

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.checkNotificationStep, expected_changes)
        subscriber2.add_step(
            subscriber2.checkNotificationStep, expected_changes)
        subscriber3.add_step(
            subscriber3.checkNotificationStep, expected_changes)
        subscriber4.add_step(
            subscriber4.checkNotificationStep, expected_changes)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.cancelSubscriptionStep)
        subscriber2.add_step(subscriber2.cancelSubscriptionStep)
        subscriber3.add_step(subscriber3.cancelSubscriptionStep)
        subscriber4.add_step(subscriber4.cancelSubscriptionStep)

        tester.add_step(tester.stopSession)
        subscriber.add_step(subscriber.stopSession)
        subscriber2.add_step(subscriber2.stopSession)
        subscriber3.add_step(subscriber3.stopSession)
        subscriber4.add_step(subscriber4.stopSession)

        tester.add_step(tester.disconnect)
        subscriber.add_step(subscriber.disconnect)
        subscriber2.add_step(subscriber2.disconnect)
        subscriber3.add_step(subscriber3.disconnect)
        subscriber4.add_step(subscriber4.disconnect)

        tm.add_tester(tester)
        tm.add_tester(subscriber)
        tm.add_tester(subscriber2)
        tm.add_tester(subscriber3)
        tm.add_tester(subscriber4)
        tm.run()

    def test_delete_default_node(self):
        tm = TestManager()

        tester = SysrepoTester("Tester", sr.SR_DS_RUNNING, False)
        subscriber = NotificationTester("Subscriber")

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.subscribeStep, "ietf-interfaces",
                            "/ietf-interfaces:interfaces")

        tester.add_step(
            tester.setItemStep, "/ietf-interfaces:interfaces/interface[name='eth0']/type", sr.Val("iana-if-type:ethernetCsmacd", sr.SR_IDENTITYREF_T))
        subscriber.add_step(subscriber.waitStep)

        tester.add_step(
            tester.setItemStep, "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/enabled", sr.Val(False, sr.SR_BOOL_T))
        subscriber.add_step(subscriber.waitStep)

        tester.add_step(tester.commitStep)
        subscriber.add_step(subscriber.waitStep)

        tester.add_step(tester.deleteItemStep,
                        "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/enabled")
        subscriber.add_step(subscriber.waitStep)

        tester.add_step(tester.commitStep)
        subscriber.add_step(subscriber.waitStep)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.checkNotificationStep, [
                            ["MODIFIED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/forwarding"]])

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.cancelSubscriptionStep)


        tester.add_step(tester.stopSession)
        subscriber.add_step(subscriber.stopSession)

        tester.add_step(tester.disconnect)
        subscriber.add_step(subscriber.disconnect)

        tm.add_tester(tester)
        tm.add_tester(subscriber)

        tm.run()

    def test_change_default_node(self):
        tm = TestManager()

        tester = SysrepoTester("Tester", sr.SR_DS_RUNNING, False)
        subscriber = NotificationTester("Subscriber")

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.subscribeStep, "ietf-interfaces",
                            "/ietf-interfaces:interfaces")

        tester.add_step(
            tester.setItemStep, "/ietf-interfaces:interfaces/interface[name='eth0']/type", sr.Val("iana-if-type:ethernetCsmacd", sr.SR_IDENTITYREF_T))
        subscriber.add_step(subscriber.waitStep)

        tester.add_step(
            tester.setItemStep, "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/forwarding", sr.Val(True, sr.SR_BOOL_T))
        subscriber.add_step(subscriber.waitStep)

        tester.add_step(tester.commitStep)
        subscriber.add_step(subscriber.waitStep)

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.checkNotificationStep, [
                            ["MODIFIED", "/ietf-interfaces:interfaces/interface[name='eth0']/ietf-ip:ipv4/forwarding"]])

        tester.add_step(tester.waitStep)
        subscriber.add_step(subscriber.cancelSubscriptionStep)


        tester.add_step(tester.stopSession)
        subscriber.add_step(subscriber.stopSession)

        tester.add_step(tester.disconnect)
        subscriber.add_step(subscriber.disconnect)

        tm.add_tester(tester)
        tm.add_tester(subscriber)

        tm.run()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

__author__ = "Antonio Paunovic <antonio.paunovic@sartura.hr>"
__copyright__ = "Copyright 2017, Deutsche Telekom AG"
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sysrepo as sr
import sys
import os
import signal
from time import sleep
from os import getpid


usr_interrupt=False
subscribe = None

def oper_get_items_cb(session, module_name, path, request_xpath, request_id, parent, private_data):
    print("\n\n ========== CALLBACK CALLED TO PROVIDE \"" + path + "\" DATA ==========\n")
    try:
        ctx = session.get_context()
        mod = ctx.get_module(module_name)

        parent.reset(sr.Data_Node(ctx, "/ietf-interfaces:interfaces-state", None, sr.LYD_ANYDATA_CONSTSTRING, 0))
        ifc = sr.Data_Node(parent, mod, "interface")
        name = sr.Data_Node(ifc, mod, "name", "eth100")
        typ = sr.Data_Node(ifc, mod, "type", "iana-if-type:ethernetCsmacd")
        oper_status = sr.Data_Node(ifc, mod, "oper-status", "down")

        ifc.reset(sr.Data_Node(parent, mod, "interface"));
        name.reset(sr.Data_Node(ifc, mod, "name", "eth101"));
        typ.reset(sr.Data_Node(ifc, mod, "type", "iana-if-type:ethernetCsmacd"));
        oper_status.reset(sr.Data_Node(ifc, mod, "oper-status", "up"));

        ifc.reset(sr.Data_Node(parent, mod, "interface"));
        name.reset(sr.Data_Node(ifc, mod, "name", "eth102"));
        typ.reset(sr.Data_Node(ifc, mod, "type", "iana-if-type:ethernetCsmacd"));
        oper_status.reset(sr.Data_Node(ifc, mod, "oper-status", "dormant"));

        ifc.reset(sr.Data_Node(parent, mod, "interface"));
        name.reset(sr.Data_Node(ifc, mod, "name", "eth105"));
        typ.reset(sr.Data_Node(ifc, mod, "type", "iana-if-type:ethernetCsmacd"));
        oper_status.reset(sr.Data_Node(ifc, mod, "oper-status", "not-present"));

    except Exception as e:
        print(e)
        return sr.SR_ERR_OK
    sys.stdout.flush()
    return sr.SR_ERR_OK

if __name__ == "__main__":
    module_name = "ietf-interfaces"
    xpath = "/ietf-interfaces:interfaces-state"
    if len(sys.argv) > 2:
        module_name = sys.argv[1]


    # connect to sysrepo
    conn = sr.Connection(sr.SR_CONN_DEFAULT)

    # start session
    sess = sr.Session(conn)

    # subscribe for changes in running config */
    subscribe = sr.Subscribe(sess)

    subscribe.oper_get_items_subscribe(module_name, xpath, oper_get_items_cb)
    
    with open("pipe_subscription_test", "w") as fifo:
        fifo.write("subscribed")
    

    sr.global_loop()

    subscribe.unsubscribe()

    sess.session_stop()

    conn = None

    print("Application exit requested, exiting.")



#!/usr/bin/env python
from __future__ import print_function

# -*- coding: utf-8 -*-
__author__ = "Rastislav Szabo <raszabo@cisco.com>, Lukas Macko <lmacko@cisco.com>"
__copyright__ = "Copyright 2016, Cisco Systems, Inc."
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from random import randint
import os, time
import unittest
import traceback
import sys


class Tester(object):
    """
    Class instance simulates the steps executed by tester.

    A step can be added by add_step function. Steps are executed in synchronized order
    with other testers.

    Attributes:
        steps (list) - list of steps to be executed
        _current_step(int) - index of the step to be executed

    """
    def __init__(self, name=None):
        self.name = name
        self.tc = unittest.TestCase('__init__')
        self.steps = []

    def setup(self):
        """Method executed before steps"""
        pass

    def cleanup(self):
        """Method executed after steps"""
        pass

    def add_step(self, step, *args):
        """Adds the step to the end of the list"""
        self.steps.append((step, args))

    def has_next(self):
        """Checks whether there is a step to be executed"""
        return self._current_step < (len(self.steps))

    def report_pid(self, pid):
        """Insert pid into notification queue"""
        if self.sub_proc is not None:
            self.sub_proc.put(pid)

    def print_with_lock(self, *args):
        """Print to stdout with acquired lock"""
        if self.lock is not None:
            self.lock.acquire()
            print(args)
            self.lock.release()
        else:
            print(args)

    def execute_step(self, index):
        step, args = self.steps[index]
        if len(args) != 0:
            step(*args)
        else:
            step()

    def run_sync(self, done, next_step, rand_sleep, id, queue):
        """run steps in sync with other tester and inform test manager"""
        if self.name is None:
            self.name = id
        #empty steps check
        if len(self.steps) == 0:
            next_step.acquire()
            queue.put((id, self.name, False))
            done.release()

        for step in range(len(self.steps)):
            err = None
            next_step.acquire()
            if rand_sleep:
                time.sleep(randint(1,1000)*0.00001)
            #self.print_with_lock('Step: ', step, ", pid: ", os.getpid())

            try:
                self.execute_step(step)
            except Exception as e:
                _, _, tb = sys.exc_info()
                traceback.print_tb(tb) # Print traceback
                print("\n\n", file=sys.stderr)
                err = e
            finally:
                self._current_step += 1
                #report to test manager if there is any step left
                if err is not None:
                    queue.put((id, self.name, err))
                elif self.has_next():
                    queue.put((id, self.name, True))
                else:
                    queue.put((id, self.name, False))
                done.release()

    def run_without_sync(self):
        """run steps independently"""
        for step in range(len(self.steps)):
            self.execute_step(step)

    def run(self, done=None, next_step=None, rand_sleep=False, lock=None, sub_proc=None, pids=None, id=-1, queue=None):
        """Executes the tester steps

            Arguments:
                done        (Semaphore) - to be acquired before step execution
                next_step   (Semaphore) - to be released on step completion
                rand_sleep  (bool)      - flag whether sleep before each step
                lock        (Lock)      - stdout synchronization
                sub_proc    (Queue)     - queue to report pids of created process in tester to be terminated by testmanager
                pids        (Array)     - pids of other testers
                id          (int)       - identification used for queue messages index of the tester pid in pids
                queue       (Queue)     - message for notification whether there is a step to be executed
        """
        self.setup()
        self.lock = lock
        self._current_step = 0
        self.pids = pids
        self.sub_proc = sub_proc

        if done is not None and next_step is not None and queue is not None:
            self.run_sync(done, next_step, rand_sleep, id, queue)
        else:
            self.run_without_sync()

        self.cleanup()

    def waitStep(self):
        """Step that can be used by tester, to do nothing"""
        pass

    def runTest(self):
        """This needs to be here to make unit test happy"""
        pass



from __future__ import print_function
from time import time

import sysrepo as sr

# Tests are created to follow already existing performance test implemented in c in tests/measure_performance.c
# They are meant to compare python bindings for sysrepo with the original and as an overall measure for
# bindings clients.

# Class stub which bundles test function and additional data for running a test.
class TestContext:
    def __init__(self, function, op_name, op_count, setup=None, teardown=None):
        self.function = function
        self.op_name = op_name
        self.op_count = op_count
        self.setup = setup
        self.teardown = teardown

# Count how many times will some test run its defining function.
OP_COUNT = 50000

# function to populate ietf-interfaces yang model
def clearDataTree(module_name, datastore):
    """
    Clear yang model.
    """

    conn = sr.Connection(sr.SR_CONN_DEFAULT)
    sess = sr.Session(conn, datastore)
    subs = sr.Subscribe(sess)

    xpath = "/" + module_name + ":*//*"

    values = sess.get_items(xpath)

    if values == None:
        return

    for i in range(values.val_cnt()):
        sess.delete_item(values.val(i).xpath())
        sess.apply_changes()



# function to populate ietf-interfaces yang model
def createDataTreeLargeIETFinterfacesModule(count, datastore):
    """
    Add data to ietf-interfaces.
    """
    conn = sr.Connection(sr.SR_CONN_DEFAULT)
    sess = sr.Session(conn, datastore)
    subs = sr.Subscribe(sess)

    for i in range(count):
        xpath = "/ietf-interfaces:interfaces/interface[name='eth" + str(i) + "']"
        xpath_ip = xpath + "/ietf-ip:ipv4/address[ip='192.168.1." + str(i) + "]"
        x_name = xpath + "/name"
        x_type = xpath + "/type"
        x_desc = xpath + "/description"
        x_enabled = xpath + "/enabled"

        x_ipv4_enabled = xpath + "/ietf-ip:ipv4/enabled"
        x_ipv4_mtu = xpath + "/ietf-ip:ipv4/mtu"
        x_ipv4_forward = xpath + "/ietf-ip:ipv4/forwarding"

        x_prefix_len = xpath_ip + "/prefix-length"

        val = sr.Val("Ethernet 0", sr.SR_STRING_T)
        sess.set_item(x_desc, val)

        val = sr.Val("iana-if-type:ethernetCsmacd", sr.SR_IDENTITYREF_T)
        sess.set_item(x_type, val)

        val = sr.Val(True, sr.SR_BOOL_T)
        sess.set_item(x_enabled, val)

        val = sr.Val(True, sr.SR_BOOL_T)
        sess.set_item(x_ipv4_enabled, val)

        val = sr.Val(1500, sr.SR_UINT16_T)
        sess.set_item(x_ipv4_mtu, val)

        val = sr.Val(False, sr.SR_BOOL_T)
        sess.set_item(x_ipv4_forward, val)

    sess.apply_changes()

# function to populate example-module yang model
def createDataTreeLargeExampleModule(count, datastore):
    """
    Add data to example-module.
    """
    conn = sr.Connection(sr.SR_CONN_DEFAULT)
    sess = sr.Session(conn, datastore)
    subs = sr.Subscribe(sess)

    for i in range(count):
        xpath = "/example-module:container/list[key1='key" + str(i) + "'][key2='key" + str(i) +"']/leaf"
        val = sr.Val("leaf" + str(i), sr.SR_STRING_T)
        sess.set_item(xpath, val)

    sess.apply_changes()


def sysrepo_setup(state):
    """
    Initialize sysrepo context which program uses
    """

    state['connection'] = sr.Connection(sr.SR_CONN_DEFAULT)
    assert state['connection'] is not None

    return

def measure(test_f, name, op_count, setup_f, teardown_f):
    """
    Function which calculates and prints running time for a single test.
    It setups and tear downs resources if necessary;
    """
    t_start = time()
    items = test_f(state, op_count, 1)
    t_end = time()

    seconds = t_end - t_start

    print("%40s| %10.0f | %10d | %13d | %10.0f | %10.2f\n" % \
          (name, (float(op_count))/ seconds, items, op_count, (float(op_count * items))/ seconds, seconds))

def perf_get_item_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."
    sess = sr.Session(conn, state['datastore'])
    assert sess is not None, "Unable to get session."

    xpath = "/example-module:container/list[key1='key0'][key2='key0']/leaf"

    for i in range(op_num):
        val = sess.get_item(xpath)
        assert val.type() is sr.SR_STRING_T, "check value type"

    return 1

# All other testing functions are similar, named after corresponding c functions.

def perf_get_item_first_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."
    sess = sr.Session(conn, state['datastore'])
    assert sess is not None, "Unable to get session."

    xpath = "/example-module:container"

    for i in range(op_num):
        val = sess.get_item(xpath)
        assert val.type() is sr.SR_CONTAINER_T, "check value type"

    return 1

def perf_get_item_with_data_load_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."

    xpath = "/example-module:container/list[key1='key0'][key2='key0']/leaf"

    for i in range(op_num):
        sess = sr.Session(conn, state['datastore'])

        val = sess.get_item(xpath)
        assert val.type() is sr.SR_STRING_T, "check value type"

    return 1

def perf_get_items_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."
    sess = sr.Session(conn, state['datastore'])
    assert sess is not None, "Unable to get session."

    xpath = "/example-module:container/list/leaf"

    for i in range(op_num):
        val = sess.get_items(xpath)

    return 1
#Not supported for now
# def perf_get_items_iter_test(state, op_num, items):

#     conn = state["connection"]
#     assert conn is not None, "Unable to get connection."
#     sess = sr.Session(conn, state['datastore'])
#     assert sess is not None, "Unable to get session."

#     xpath = "/example-module:container/list/leaf"

#     count = 0
#     for i in range(op_num):
#         it = sess.get_items_iter(xpath)
#         assert it is not None, "Iterator not found"
#         while True:
#             val = sess.get_item_next(it)
#             if val == None: break
#             count = count + 1

#     return count

#Not supported for now
def perf_get_ietf_interfaces_test(state, op_num, items):

     conn = state["connection"]
     assert conn is not None, "Unable to get connection."
     sess = sr.Session(conn, state['datastore'])
     assert sess is not None, "Unable to get session."

     xpath = "/ietf-interfaces:interfaces//."

     count = 0
     for i in range(op_num):
         tree = sess.get_data(xpath , 0)
         assert tree is not None, "Iterator not found"
         temp = tree.tree_for()
         count += len(temp)

     return count

def perf_get_subtree_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."
    sess = sr.Session(conn, state['datastore'])
    assert sess is not None, "Unable to get session."

    xpath = "/example-module:container/list[key1='key0'][key2='key0']/leaf"

    for i in range(op_num):
        tree = sess.get_subtree(xpath)
        assert tree is not None, "check if empty"

    return 1

def perf_get_subtree_with_data_load_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."

    xpath = "/example-module:container/list[key1='key0'][key2='key0']/leaf"

    for i in range(op_num):
        sess = sr.Session(conn, state['datastore'])
        tree = sess.get_subtree(xpath)
        assert tree is not None, "check if empty"

    return 1

def perf_get_subtrees_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."
    sess = sr.Session(conn, state['datastore'])
    assert sess is not None, "Unable to get session."

    xpath = "/example-module:container/list[key1='key0'][key2='key0']/leaf"
    for i in range(op_num):
        trees = sess.get_subtree(xpath).tree_for()
        assert trees[0] is not None, "check if empty"
    return len(trees)

def get_nodes_cnt(trees):

    count = 0

    for i in range(len(trees)):
        node = trees[0]
        temp = node.tree_for()
        count+= len(temp)

    return count

def perf_get_ietf_intefaces_tree_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."
    sess = sr.Session(conn, state['datastore'])
    assert sess is not None, "Unable to get session."
    count = 0

    xpath = "/ietf-interfaces:interfaces/."

    for i in range(op_num):
        trees = sess.get_subtree(xpath).tree_for()
        assert trees[0] is not None, "check if empty"
        count = count + get_nodes_cnt(trees)

    return count

def perf_set_delete_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."
    sess = sr.Session(conn, state['datastore'])
    assert sess is not None, "Unable to get session."

    xpath = "/example-module:container/list[key1='set_del'][key2='set_1']/leaf"

    for i in range(op_num):
        val = sr.Val("Leaf", sr.SR_STRING_T)
        sess.set_item(xpath, val)
        sess.apply_changes()
        sess.delete_item(xpath)
        sess.apply_changes()

    return 1 * 3 * 2

def perf_set_delete_100_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."
    sess = sr.Session(conn, state['datastore'])
    assert sess is not None, "Unable to get session."

    xpath = "/example-module:container/list[key1='set_del'][key2='set_1']/leaf"

    sess.apply_changes()

    for i in range(op_num):
        sess.apply_changes()
        for j in range(100):
            xpath = "/example-module:container/list[key1='set_del'][key2='set_" + str(j) + "']/leaf"
            val = sr.Val("Leaf", sr.SR_STRING_T)
            sess.set_item(xpath, val)
        sess.apply_changes()
        for j in range(100):
            xpath = "/example-module:container/list[key1='set_del'][key2='set_" + str(j) + "']/leaf"
            sess.delete_item(xpath)
        sess.apply_changes()
       

    return 100 * 1 * 3 * 2

def perf_commit_test(state, op_num, items):

    conn = state["connection"]
    assert conn is not None, "Unable to get connection."
    sess = sr.Session(conn, state['datastore'])
    assert sess is not None, "Unable to get session."

    xpath = "/example-module:container/list[key1='key0'][key2='key0']/leaf"
    sess.apply_changes()
    for i in range(op_num):
        if (i % 2 == 0):
            val = sr.Val("Leaf", sr.SR_STRING_T)
            sess.set_item(xpath, val)
        else:
            sess.delete_item(xpath)

        sess.apply_changes()

    return 1

def print_measure_header(title):
    print ("\n\n\t\t%s" % (title)) ,
    print ("\n%-40s| %10s | %10s | %13s | %10s | %10s.\n" % ("Operation", "ops/sec", "items/op", "ops performed", "items/sec", "test time")),
    print ("---------------------------------------------------------------------------------------------------\n"),

def test_perf(ts, test_count, title, selection):
    print_measure_header(title)

    for i in range(test_count):
        if -1 == selection or i == selection:
            t = ts[i]
            measure(t.function, t.op_name, t.op_count, t.setup, t.teardown)

if __name__ == "__main__":

    op_count = 5000

    tests = [TestContext(perf_get_item_test, "Get item one leaf", op_count),
            #  TestContext(perf_get_item_first_test, "Get item first leaf", op_count),
            #  TestContext(perf_get_item_with_data_load_test, "Get item (including session start)", op_count),
            #  TestContext(perf_get_items_test, "Get all items of a list", op_count),
            #  TestContext(perf_get_ietf_interfaces_test, "Get subtrees ietf-if config", op_count),
            #  TestContext(perf_get_subtree_test, "Get subtree one leaf", op_count),
            #  TestContext(perf_get_subtree_with_data_load_test, "Get subtree (including session start)", op_count),
             TestContext(perf_get_subtrees_test, "Get subtree all leaf", op_count),
            #  TestContext(perf_get_ietf_intefaces_tree_test, "Get subtrees ietf-if config", op_count),
            #  TestContext(perf_set_delete_test, "Set & delete one list", op_count),
            #  TestContext(perf_set_delete_100_test, "Set & delete 100 lists", op_count),
            #  TestContext(perf_commit_test, "Commit one leaf change", op_count),
    ]
    
    state = {}
    state['datastore'] = sr.SR_DS_STARTUP
    sysrepo_setup(state)

    elements = [1, 20, 100]
    datastores = [sr.SR_DS_STARTUP]

    try:
        for el in elements:
            for datastore in datastores:
                state['datastore'] = datastore
                clearDataTree("ietf-interfaces", state['datastore'])
                clearDataTree("example-module", state['datastore'])
                createDataTreeLargeIETFinterfacesModule(el, state['datastore'])
                createDataTreeLargeExampleModule(el, state['datastore'])
                if (state["datastore"] == sr.SR_DS_RUNNING):
                    test_perf(tests, len(tests), "Data file " + str(el) + " list instance in datastore running", -1)
                elif (state["datastore"] == sr.SR_DS_STARTUP):
                    test_perf(tests, len(tests), "Data file " + str(el) + " list instance in datastore startup", -1)
    except Exception as e:
        print (e)

    # clean
    try:
        clearDataTree("ietf-interfaces", state['datastore'])
    except Exception as e:
        print (e)

    print ("End")



#!/usr/bin/env python
from __future__ import print_function

__author__ = "Mislav Novakovic <mislav.novakovic@sartura.hr>"
__copyright__ = "Copyright 2018, Deutsche Telekom AG"
__license__ = "Apache 2.0"

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This sample application demonstrates use of Python programming language bindings for sysrepo library.
# Original c application was rewritten in Python to show similarities and differences
# between the two.
#
# Most notable difference is in the very different nature of languages, c is weakly statically typed language
# while Python is strongly dynamiclally typed. Python code is much easier to read and logic easier to comprehend
# for smaller scripts. Memory safety is not an issue but lower performance can be expected.
#
# The original c implementation is also available in the source, so one can refer to it to evaluate trade-offs.


import sysrepo as sr
import sys
import gc

def oper_get_items_cb1(session, module_name, path, request_xpath, request_id, parent, private_data):
    print ("\n\n ========== CALLBACK CALLED TO PROVIDE \"" + path + "\" DATA ==========\n")
    try:
        ctx = session.get_context()
        mod = ctx.get_module(module_name)

        parent.reset(sr.Data_Node(ctx, "/ietf-interfaces:interfaces-state", None, sr.LYD_ANYDATA_CONSTSTRING, 0))
        ifc = sr.Data_Node(parent, mod, "interface")
        name = sr.Data_Node(ifc, mod, "name", "eth100")
        typ = sr.Data_Node(ifc, mod, "type", "iana-if-type:ethernetCsmacd")
        oper_status = sr.Data_Node(ifc, mod, "oper-status", "down")

        ifc.reset(sr.Data_Node(parent, mod, "interface"));
        name.reset(sr.Data_Node(ifc, mod, "name", "eth101"));
        typ.reset(sr.Data_Node(ifc, mod, "type", "iana-if-type:ethernetCsmacd"));
        oper_status.reset(sr.Data_Node(ifc, mod, "oper-status", "up"));

        ifc.reset(sr.Data_Node(parent, mod, "interface"));
        name.reset(sr.Data_Node(ifc, mod, "name", "eth102"));
        typ.reset(sr.Data_Node(ifc, mod, "type", "iana-if-type:ethernetCsmacd"));
        oper_status.reset(sr.Data_Node(ifc, mod, "oper-status", "dormant"));

        ifc.reset(sr.Data_Node(parent, mod, "interface"));
        name.reset(sr.Data_Node(ifc, mod, "name", "eth105"));
        typ.reset(sr.Data_Node(ifc, mod, "type", "iana-if-type:ethernetCsmacd"));
        oper_status.reset(sr.Data_Node(ifc, mod, "oper-status", "not-present"));

    except Exception as e:
        print (e)
        return sr.SR_ERR_OK
    return sr.SR_ERR_OK

def oper_get_items_cb2(session, module_name, path, request_xpath, request_id, parent, private_data):
    print ("\n\n ========== CALLBACK CALLED TO PROVIDE \"" + path + "\" DATA ==========\n")
    try:
        ctx = session.get_context()
        mod = ctx.get_module(module_name)

        stats = sr.Data_Node(parent, mod, "statistics")
        dis_time = sr.Data_Node(stats, mod, "discontinuity-time", "2019-01-01T00:00:00Z")
        in_oct = sr.Data_Node(stats, mod, "in-octets", "22")

    except Exception as e:
        print (e)
        return sr.SR_ERR_OK
    return sr.SR_ERR_OK

# Notable difference between c implementation is using exception mechanism for open handling unexpected events.
# Here it is useful because `Conenction`, `Session` and `Subscribe` could throw an exception.
try:
    module_name = "ietf-interfaces"
    if len(sys.argv) > 1:
        module_name = sys.argv[1]
    else:
        print ("\nYou can pass the module name to be subscribed as the first argument")

    # connect to sysrepo
    conn = sr.Connection(sr.SR_CONN_DEFAULT)

    # start session 
    sess = sr.Session(conn)

    # subscribe for changes in running config */
    subscribe = sr.Subscribe(sess)

    print ("\nApplication will provide data of " + module_name + " module\n")

    #try:
    subscribe.oper_get_items_subscribe(module_name, "/ietf-interfaces:interfaces-state", oper_get_items_cb1)
    #except Exception as e:
    #    print (e)

    sr.global_loop()

    subscribe.unsubscribe()

    sess.session_stop()

    conn=None

    print ("Application exit requested, exiting.\n")

except Exception as e:
    print (e)
